/*
 * ST7735.c
 *
 * Created: 07/07/2014 16:44:17
 *  Author: Ricardo
 */ 

#define F_CPU 20000000UL

#include <avr/io.h>
#include <util/delay.h>
#include "SPI.h"
#include "ST7735.h"


void ST7735_write_command(uint8_t command)
{
	RS_LOW;
	SS_LOW;
	
	SPI_data(command);
	
	SS_HIGH;
}

void ST7735_write_data(uint8_t data)
{
	RS_HIGH;
	SS_LOW;
	
	SPI_data(data);
	
	SS_HIGH;
}

void ST7735_display_init()
{
	ST7735_write_command(0x01); // software reset
	_delay_ms(150);
	
	ST7735_write_command(0x11);  // out of sleep mode
	_delay_ms(500);
	
	ST7735_write_command(0xB1); // frame control normal mode
	ST7735_write_data(0x01);
	ST7735_write_data(0x2C);
	ST7735_write_data(0x2D);
	
	ST7735_write_command(0xB2); // frame control idle mode
	ST7735_write_data(0x01);
	ST7735_write_data(0x2C);
	ST7735_write_data(0x2D);
	
	ST7735_write_command(0xB3); // frame control partial mode
	ST7735_write_data(0x01);
	ST7735_write_data(0x2C);
	ST7735_write_data(0x2D);
	ST7735_write_data(0x01);
	ST7735_write_data(0x2C);
	ST7735_write_data(0x2D);
	
	ST7735_write_command(0xB4); // display inversion control
	ST7735_write_data(0x07);
	
	ST7735_write_command(0xC0); // power control
	ST7735_write_data(0xA2);
	ST7735_write_data(0x02);
	ST7735_write_data(0x84);
	
	ST7735_write_command(0xC1); // power control 2
	ST7735_write_data(0xc5);
	
	ST7735_write_command(0xC2); // power control 3
	ST7735_write_data(0x0A);
	ST7735_write_data(0x00);
	
	ST7735_write_command(0xC3); // power control 4
	ST7735_write_data(0x8A);
	ST7735_write_data(0x2A);
	
	ST7735_write_command(0xC4); // power control 5
	ST7735_write_data(0x8A);
	ST7735_write_data(0xEE);
	
	ST7735_write_command(0xC5); // power control 6
	ST7735_write_data(0x0E);
	
	ST7735_write_command(0x20); // no display inversion
	
	ST7735_write_command(0x36); // memory access control direction
	ST7735_write_data(0xC8);
	
	ST7735_write_command(0x3A); // set 16 bit color mode
	ST7735_write_data(0x05);
	
	ST7735_write_command(0x2A); // column address set
	ST7735_write_data(0x00);
	ST7735_write_data(0x00);
	ST7735_write_data(0x00);
	ST7735_write_data(0x7F);
	
	ST7735_write_command(0x2A); // row address set
	ST7735_write_data(0x00);
	ST7735_write_data(0x00);
	ST7735_write_data(0x00);
	ST7735_write_data(0x9F);
	
	ST7735_write_command(0xE0); // other stuff
	ST7735_write_data(0x02);
	ST7735_write_data(0x1C);
	ST7735_write_data(0x07);
	ST7735_write_data(0x12);
	ST7735_write_data(0x37);
	ST7735_write_data(0x32);
	ST7735_write_data(0x29);
	ST7735_write_data(0x2D);
	ST7735_write_data(0x29);
	ST7735_write_data(0x25);
	ST7735_write_data(0x2B);
	ST7735_write_data(0x39);
	ST7735_write_data(0x00);
	ST7735_write_data(0x01);
	ST7735_write_data(0x03);
	ST7735_write_data(0x10);
	
	ST7735_write_command(0xE1); // yet more stuff
	ST7735_write_data(0x03);
	ST7735_write_data(0x1D);
	ST7735_write_data(0x07);
	ST7735_write_data(0x06);
	ST7735_write_data(0x2E);
	ST7735_write_data(0x2C);
	ST7735_write_data(0x29);
	ST7735_write_data(0x2D);
	ST7735_write_data(0x2E);
	ST7735_write_data(0x2E);
	ST7735_write_data(0x37);
	ST7735_write_data(0x3F);
	ST7735_write_data(0x00);
	ST7735_write_data(0x00);
	ST7735_write_data(0x02);
	ST7735_write_data(0x10);
	
	ST7735_write_command(0x13); // normal display on
	_delay_ms(10);
	
	ST7735_write_command(0x29); // main screen on
	_delay_ms(100);
	
	ST7735_write_command(0x36); // memory access control direction
	ST7735_write_data(0xC0);
}

void ST7735_init()
{
	DDRB |= (1<<PB1); //RS as output
	
	SPI_init();
	ST7735_display_init();
	
	ST7735_fill(COLOR_BLACK);
}

void ST7735_set_addr_window(uint8_t x0, uint8_t y0, uint8_t x1, uint8_t y1)
{
	ST7735_write_command(0x2A); // column address start
	ST7735_write_data(0x00);
	ST7735_write_data(x0);
	ST7735_write_data(0x00);
	ST7735_write_data(x1);

	ST7735_write_command(0x2B); // row address start
	ST7735_write_data(0x00);
	ST7735_write_data(y0);
	ST7735_write_data(0x00);
	ST7735_write_data(y1);
	
	ST7735_write_command(0x2C); // write to ram
}

void ST7735_fill(uint16_t color)
{
  uint8_t x, y;
  ST7735_set_addr_window(0, 0, 127, 159);
  
  for (x = 0; x < 128; x++)
  {
    for (y = 0; y < 160; y++) 
    {
      ST7735_write_data(color >> 8);    
      ST7735_write_data(color);  
    }
  }
}

void ST7735_draw_pixel(uint16_t x, uint16_t y, uint16_t color)
{
  ST7735_set_addr_window(x,y,x+1,y+1);
  ST7735_write_data(color >> 8);  
  ST7735_write_data(color);
}